/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.structures.*;

class com.bourre.structures.AbstractGrid
{
	private var _aContent : Array;
	private var _vSize:Point;
	
	private function AbstractGrid (nX:Number, nY:Number) 
	{
		if (nX <0 || nY <0 || nX == undefined || nY == undefined) 
		{
			trace('**Error** invalid parameters for building Grid : [' + nX + ', ' + nY + ']');
		}

		_vSize = new Point(nX, nY);
		initContent();
	}
	
	public function initContent() : Void 
	{
		_aContent  = new Array(_vSize.x);
		for (var x = 0; x < _vSize.x; x++) _aContent[x] = new Array(_vSize.y);
	}
	
	public function isGridCoords(v:Point) : Boolean
	{
		return (v.x >= 0 && v.y >= 0 && v.x < _vSize.x && v.y < _vSize.y);
	}
	
	public function checkGridCoords(v) : Void
	{
		if (!isGridCoords(v)) trace("**Error** invalid grid coordinates : " + v);
	}
	
	public function getArea() : Number 
	{
		return _vSize.x * _vSize.y; 
	}
	
	public function getSize() : Point 
	{ 
		return _vSize; 
	}

	public function getCoordinates (n:Number) : Point 
	{
		var nY:Number = Math.floor(n / _vSize.x);
		return new Point(n - (nY * _vSize.x), nY);
	}
	
	public function getIntFromVector (v:Point) : Number 
	{ 
		return v.x + ( v.y * _vSize.x ); 
	}
	
	public function getRow(n:Number) : Array
	{
		checkGridCoords( new Point(0, n) );
		var a:Array = new Array();
		var l:Number = getSize().x;
		for (var i:Number = 0; i<l; i++)
		{
			a.push( getVal( new Point(i, n)) );
		}
		return a;
	}
	
	public function getColumn(n:Number) : Array
	{
		checkGridCoords( new Point(n, 0) );
		return _aContent[n].concat();
	}
	
	public function getRandomCoords() : Point
	{
		return new Point(
					Math.round(Math.random() * (getSize().x - 1) ),
					Math.round(Math.random() * (getSize().y - 1) )
					);
	}
	
	public function searchForOne(v) : Point
	{ 
		var i = getIterator();
		while(i.hasNext()) if (i.next() == v) return(i);
	}
	
	public function searchForAll(v) : Array
	{
		var a:Array = new Array();
		var i = getIterator();
		while(i.hasNext()) if (i.next() == v) a.push(i);
		return a;
	}
	
	private function _fillWith(o) : Void
	{
		var i = getIterator();
		while(i.hasNext()) 
		{
			i.next();
			_aContent[i.x][i.y] = o;
		}
	}
	
	public function getIterator() : GridIterator
	{
		return new GridIterator(this);
	}
	
	public function toString() : String 
	{
		var r:String = "\n";
		var s:String = "Width and Heigth : " + _vSize + r;
		s += "Content : " + "[" + _aContent + "]"  + r; 
		return s;
	}
	
	// virtual methods
	public var getVal:Function;
	public var setVal:Function;
}