/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.EventType;
import com.bourre.events.IEvent;

class com.bourre.events.BasicEvent
	implements IEvent
{
	private var _e:EventType;
	private var _oT;
	
	/**
	 * BasicEvent is the basical event structure to work with EventBroadcaster.
	 * @example <code>var e:BasicEvent = new BasicEvent(MyClass.onSomething, this);</code>
	 * @param e event type (event name).
	 * @param oT event target.
	 */
	public function BasicEvent(e:EventType, oT) 
	{
		_e = e;
		_oT = oT;
	}
	
	/**
	 * Get event type (event name).
	 * @example <code>var o:EventType = e.getType();</code>
	 * @return EventType
	 */
	public function getType() : EventType
	{ 
		return _e; 
	}
	
	/**
	 * Set event type (event name).
	 * @example <code>e.setType(MyClass.onSomething);</code>
	 * @param e EventType.
	 */
	public function setType(e:EventType) : Void 
	{ 
		_e = e; 
	}
	
	/**
	 * Get event target.
	 * @example <code>var o:ITarget = e.getTarget();</code>
	 * @return event target. Loose typing for interfaces support.
	 */
	public function getTarget()
	{ 
		return _oT; 
	}
	
	/**
	 * Set event target.
	 * @example <code>e.setTarget(ITarget);</code>
	 * @param e event target. Loose typing for interfaces support.
	 */
	public function setTarget(oT) : Void 
	{ 
		_oT = oT; 
	}
	
	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return '[BasicEvent' + HashCodeFactory.getKey( this ) + ' : ' + getType() + ']';
	}
}