/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;

class com.bourre.events.BooleanEvent extends BasicEvent
{
	private var _b:Boolean;
	
	/**
	 * Class inherited from BasicEvent with Boolean property bonus.<br><br>
	 * <strong>Usage :</strong> <em>var e:BooleanEvent = new BooleanEvent(MyClass.onSomething, true);</em>
	 * @param e event type (event name).
	 * @param b Boolean.
	 */
	public function BooleanEvent(e:EventType, b:Boolean)
	{
		super(e);
		_b = b;
	}
	
	/**
	 * Get Boolean value.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = e.getBoolean();</em>
	 * @return Boolean
	 */
	public function getBoolean() : Boolean
	{
		return _b;
	}
	
	/**
	 * Set Boolean value.<br><br>
	 * <strong>Usage :</strong> <em>e.setBoolean(true);</em>
	 * @param b Boolean
	 */
	public function setBoolean(b:Boolean) : Void
	{
		_b = b;
	}
	
	/**
	 * Get Boolean value.<br><br>
	 * <strong>Usage :</strong> <em>if(e.isTrue) doSomething();</em>
	 * @return Boolean
	 */
	public function get isTrue() : Boolean
	{
		return _b;
	}
	
	public function toString() : String
	{
		return 'BooleanEvent' + HashCodeFactory.getKey( this ) + ' : ' + getType() + ', ' + getBoolean();
	}
}