/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.core.HashCodeFactory;
import com.bourre.events.BubbleEvent;
import com.bourre.events.EventBroadcaster;
import com.bourre.events.IEvent;
import com.bourre.events.ListenerArray;

class com.bourre.events.BubbleEventBroadcaster
	extends EventBroadcaster
{
	public var parent:BubbleEventBroadcaster;
	
	/**
	 * 
	 */
	public function BubbleEventBroadcaster( owner, parent:BubbleEventBroadcaster ) 
	{
		super( owner );
		this.parent = parent? parent : null;
	}

	/**
	 * To receive bubble events from a child.
	 * @param child Object that will send bubble events.
	 */
	public function addChild( child : BubbleEventBroadcaster ) : Void
	{
		child.parent = this;
	}
	
	/**
	 * To stop receiving bubble events from a child.
	 * @param child Object that will stop to send bubble events.
	 */
	public function removeChild( child : BubbleEventBroadcaster ) : Void
	{
		child.parent = null;
	}
	
	/**
	 * Broadcast event to suscribed listeners if event propagation property is set to true.
	 * The event is bubbled to the parent if event bubble property is set to true.
	 * @param e Event to broadcast or bubble.
	 */
	public function broadcastEvent( e : BubbleEvent ) : Void
	{
		if ( e.propagation ) super.broadcastEvent( e );
		if ( e.bubbles && parent )
		{
			_broadcastTo( parent, e );
			parent.broadcastEvent( e );
		}

	}
	
	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return 'BubbleEventBroadcaster' + HashCodeFactory.getKey( this );
	}
	
	// Overwrite
	private function _broadcast(aL:ListenerArray, e:BubbleEvent) : Void
	{
		var l:Number = aL.length;
		while( --l > -1 )
		{
			if (!e.propagation) 
			{
				break;
			} else
			{
				_broadcastTo(aL[l], e);
			}
			
		}
	}
	
	private function _broadcastTo(o, e:IEvent ) : Void
	{
		var sType:String = typeof(o);
		if (sType == "object" || sType == "movieclip") 
		{
			if (o.handleEvent != undefined)
			{
				o.handleEvent(e);
			} else
			{
				o[String(e.getType())](e);
			}
		} else
		{
			o.apply( this, [e] );
		}
	}
}