/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;

dynamic class com.bourre.events.DynBasicEvent extends BasicEvent
{
	public function DynBasicEvent(s:String, oT)
	{
		super( new EventType(s), oT);
	}
	
	/**
	 * Get event type (event name).<br><br>
	 * <strong>Usage :</strong> <em>var o:EventType = e.type;</em>
	 * @return EventType
	 */
	public function get type() : String 
	{ 
		return _e.toString(); 
	}
	
	/**
	 * Set event type (event name).<br><br>
	 * <strong>Usage :</strong> <em>e.type = MyClass.onSomething;</em>
	 * @param e EventType.
	 */
	public function set type(s:String) : Void 
	{ 
		_e = new EventType(s); 
	}
	
	/**
	 * Get event target.<br><br>
	 * <strong>Usage :</strong> <em>var o:ITarget = e.target;</em>
	 * @return event target. Loose typing for interfaces support.
	 */
	public function get target() : Object 
	{ 
		return _oT; 
	}
	
	/**
	 * Set event target.<br><br>
	 * <strong>Usage :</strong> <em>e.target = ITarget;</em>
	 * @param e event target. Loose typing for interfaces support.
	 */
	public function set target(oT) : Void 
	{ 
		_oT = oT; 
	}
	
	public function toString() : String
	{
		return '[DynBasicEvent' + HashCodeFactory.getKey( this ) + ' : ' + this.type + ']';
	}
}