/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.events.EventType;
import com.bourre.transitions.IFrameBeacon;
import com.bourre.transitions.IFrameListener;

class com.bourre.transitions.FPSBeacon
	implements IFrameBeacon
{
	private static var _oB:FPSBeacon;
	
	private var addListener:Function;
	private var removeListener:Function;
	private var broadcastMessage:Function;
	private var _listeners:Array;
	
	private var _mc:MovieClip;
	private var _f:Function;
	
	public static var onEnterFrameEVENT:EventType = new EventType('onEnterFrame');
	
	private function FPSBeacon()
	{
		AsBroadcaster.initialize(this);
		_mc = _level0.createEmptyMovieClip ("__mcBeacon", 9997);
		_f = Delegate.create(this, broadcastMessage, onEnterFrameEVENT);
	}
	
	/**
	 * Get FPSBeacon instance (Singleton pattern).<br><br>
	 * <strong>Usage :</strong> <em>var oFB:FPSBeacon = FPSBeacon.getInstance();</em>
	 */
	public static function getInstance() : FPSBeacon
	{
		if (!_oB) _oB = new FPSBeacon();
		return _oB;
	}
	
	/**
	 * Destroy FPSBeacon instance (Singleton pattern).<br><br>
	 * <strong>Usage :</strong> <em>FPSBeacon.release();</em>
	 */
	public static function release() : Void
	{
		_oB.stop();
		_oB._mc.removeMovieClip();
		delete _oB;
	}
	
	/**
	 * Starts the process.<br><br>
	 * <strong>Usage :</strong> <em>FPSBeacon.getInstance().start();</em>
	 */
	public function start() : Void
	{
		_mc.onEnterFrame = _f;
	}
	
	/**
	 * Stops the process.<br><br>
	 * <strong>Usage :</strong> <em>FPSBeacon.getInstance().stop();</em>
	 */
	public function stop() : Void
	{
		delete _mc.onEnterFrame;
	}
	
	/**
	 * To query if FPSBeacon's running.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = FPSBeacon.getInstance().isPlaying;</em>
	 * @return Boolean.
	 */
	public function isPlaying() : Boolean
	{
		return _mc.onEnterFrame == _f;
	}
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function addFrameListener(oL:IFrameListener) : Void
	{
		if (_listeners.length < 1) start();
		addListener(oL);
	}
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function removeFrameListener(oL:IFrameListener) : Void
	{
		removeListener(oL);
		if (_listeners.length < 1) this.stop();
	}
	
	public function toString() : String
	{
		return 'FrameBeacon' + HashCodeFactory.getKey( this );
	}
}