/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.commands.Command;
import com.bourre.core.HashCodeFactory;
import com.bourre.data.collections.Map;
import com.bourre.events.EventBroadcaster;
import com.bourre.events.EventType;
import com.bourre.events.IEvent;
import com.bourre.log.Logger;
import com.bourre.log.LogLevel;

class com.bourre.events.FrontController
{
	public var _oEB:EventBroadcaster;
	private var _a:Map;
	
	public function FrontController(oEB:EventBroadcaster)
	{
		_a = new Map();
		_oEB = oEB;
		if (_oEB == undefined) _oEB = EventBroadcaster.getInstance();
	}

	public function addEventListener(e:EventType, oL) : Void
	{
		_oEB.addEventListener(e, oL);
	}
	
	public function removeEventListener(e:EventType, oL) : Void
	{
		_oEB.removeEventListener(e, oL);
	}
	
	public function getCommand(s:String) : Command
	{
		if (!_a.containsKey( s )) Logger.LOG("**Error** " + toString() + ".getCommand() can't retrieve '" + s + "' command.", LogLevel.ERROR);
		return _a.get( s );
	}
	
	public function push(s:String, oC:Command) : Void
	{
		_a.put(s, oC);
		_oEB.addEventListener(s, this);
	}
	
	public function remove(s:String) : Void
	{
		_a.remove( s );
		_oEB.removeEventListener(s, this);
	}
	
	public function handleEvent(e:IEvent) : Void
	{
		_executeCommand(e);
	}
	
	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return 'FrontController' + HashCodeFactory.getKey( this );
	}
	
	//
	private function _executeCommand(e:IEvent) : Void
	{
		var t:String = (e.getType() == undefined) ? Object(e).type.toString() : e.getType().toString();
		getCommand( t ).execute();
	}
}