﻿/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.data.libs.AbstractLib;
import com.bourre.data.libs.GraphicLibEvent;
import com.bourre.events.EventType;
import com.bourre.log.LogLevel;
import com.bourre.log.Logger;

class com.bourre.data.libs.GraphicLib extends AbstractLib
{
	private var _mcContainer:MovieClip;
	private var _mcLib:MovieClip;
	private var _bAutoShow:Boolean;
	
	public static var onLoadInitEVENT:EventType = AbstractLib.onLoadInitEVENT;
	public static var onLoadProgressEVENT:EventType = AbstractLib.onLoadProgressEVENT;
	public static var onTimeOutEVENT:EventType = AbstractLib.onTimeOutEVENT;
	
	public function GraphicLib(mcTarget:MovieClip, nDepth:Number, bAutoShow:Boolean)
	{
		super();
		
		if (mcTarget == undefined)
		{
			Logger.LOG( "**Error** " + toString() + " MovieClip target is undefined.", LogLevel.ERROR );
		}
		
		if (nDepth == undefined) nDepth = 1;
		_mcContainer = mcTarget.createEmptyMovieClip("__container" + HashCodeFactory.getNextName(), nDepth);
		HashCodeFactory.getKey( _mcContainer );
		_bAutoShow = (bAutoShow == undefined) ? true : bAutoShow;
	}
	
	public function initEventSource() : Void
	{
		_e = new GraphicLibEvent(null, this);
	}
	
	public function load(sURL:String) : Void
	{
		if (sURL != undefined) setURL( sURL );
		
		release();
		setContent( _mcContainer.createEmptyMovieClip("__mcLib", 1) );
		getContent().loadMovie( super.getURL() );
		hide();
		
		super.load();
	}
	
	public function onLoadInit() : Void
	{
		super.onLoadInit();
		if (_bAutoShow) show();
	}
	
	public function show() : Void
	{
		_mcContainer._visible = true;
	}
	
	public function hide() : Void
	{
		_mcContainer._visible = false;
	}
	
	public function set autoShow(b:Boolean) : Void
	{
		_bAutoShow = b;
	}
	
	public function release() : Void
	{
		getContent().removeMovieClip();
		_mcContainer._visible = _bAutoShow;
		super.release();
	}

	public function getContent() : MovieClip
	{
		return super.getContent();
	}
	
	public function setContent(mc:MovieClip) : Void
	{
		super.setContent( mc );
	}
	
	public function getView() : MovieClip
	{
		return super.getContent();
	}
	
	public function toString() : String
	{
		return 'GraphicLib' + HashCodeFactory.getKey( this );
	}
}