/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.structures.*;
import com.bourre.data.iterator.*;

class com.bourre.structures.GridIterator extends Point
							implements Iterable
{
	private var _g:AbstractGrid;
	private var _n:Number;
	private var _l:Number;
	
	/**
	 * Usage : <em>var i:GridIterator = new GridIterator(myGrid);</em>
	 * @param g 				target Grid for the iterator.
	 */
	public function GridIterator(g:AbstractGrid)
	{
		super(undefined, undefined);
		_g = g;
		_n = -1;
		_l = _g.getArea() - 1;
	}
	
	/**
	 * To check if the next iterator position exists<br><br>
	 * <strong>Usage :</strong> <em>while(myIterator.hasNext())</em>
	 * @return Boolean that indicates if the next iterator position exists.
	 */
	public function hasNext() : Boolean 
	{
		return _n < _l; 
	}

	/**
	 * To increment the iterator position and get the next object value<br><br>
	 * <strong>Usage :</strong> <em>var o:Object = myIterator.next();</em>
	 * @return No return type has been specified to authorize Grid polymorphism.
	 */
	public function next()
	{
		reset(_g.getCoordinates(++_n));
		return  _g.getVal(this);
	}
	
	/**
	 * To get index value that indicates the current tile number.<br><br>
	 * <strong>Usage :</strong> <em>var nIndex:Number = myIterator.getIndex();</em>
	 * @return Number that indicates the current position of the iterator.
	 */
	public function getIndex() : Number 
	{ 
		return _n; 
	}
}