/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.transitions.IFrameListener;

interface com.bourre.transitions.IFrameBeacon 
{
	/**
	 * Starts the process.<br><br>
	 * <strong>Usage :</strong> <em>MSBeacon.getInstance().start();</em>
	 */
	public function start() : Void;
	
	/**
	 * Stops the process.<br><br>
	 * <strong>Usage :</strong> <em>MSBeacon.getInstance().stop();</em>
	 */
	public function stop() : Void;
	
	/**
	 * To query if MSBeacon's running.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = MSBeacon.getInstance().isPlaying;</em>
	 * @return Boolean.
	 */
	public function isPlaying() : Boolean;
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function addFrameListener(oL:IFrameListener) : Void;
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function removeFrameListener(oL:IFrameListener) : Void;
}