/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.core.HashCodeFactory;
import com.bourre.log.LogEvent;
import com.bourre.log.LogLevel;
import com.bourre.log.LogListener;

import LuminicBox.Log.ConsolePublisher;
import LuminicBox.Log.Logger;

class com.bourre.utils.LuminicTracer
	implements LogListener
{
	private static var _oInstance:LuminicTracer;
	
	private var _oLumiLog:Logger;
	
	private function LuminicTracer()
	{
		_oLumiLog = new LuminicBox.Log.Logger("logger");
		_oLumiLog.addPublisher( new ConsolePublisher() );
	}
	
	/**
	 * static methods
	 */
	public static function getInstance() : LuminicTracer
	{
		if (!LuminicTracer._oInstance) LuminicTracer._oInstance = new LuminicTracer();
		return LuminicTracer._oInstance;
	}
	
	public function onLog( e : LogEvent ) : Void 
	{
		switch( e.level )
		{
			case LogLevel.DEBUG :
				_oLumiLog.debug( e.content );
				break;
			case LogLevel.INFO :
				_oLumiLog.info( e.content );
				break;
			case LogLevel.WARN :
				_oLumiLog.warn( e.content );
				break;
			case LogLevel.ERROR :
				_oLumiLog.error( e.content );
				break;
			case LogLevel.FATAL :
				_oLumiLog.fatal( e.content );
				break;
		}
	}

	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return 'LuminicTracer' + HashCodeFactory.getKey( this );
	}
}