/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.core.HashCodeFactory;
import com.bourre.events.EventType;
import com.bourre.transitions.IFrameBeacon;
import com.bourre.transitions.IFrameListener;

class com.bourre.transitions.MSBeacon
	implements IFrameBeacon
{
	private static var _oB:MSBeacon;
	
	private var addListener:Function;
	private var removeListener:Function;
	private var broadcastMessage:Function;
	private var _listeners:Array;
	
	private var _nID:Number;
	private var _bIsP:Boolean;
	private var _nFrameRate:Number;
	
	public static var DEFAULTFRAMERATE:Number = 10;
	public static var onEnterFrameEVENT:EventType = new EventType('onEnterFrame');

	public function MSBeacon( nFrameRate:Number )
	{
		AsBroadcaster.initialize(this);
		_bIsP = false;
		_nFrameRate = nFrameRate;
	}
	
	public function fireFrameEvent()
	{
		this.broadcastMessage(onEnterFrameEVENT);
		updateAfterEvent();
	}
	
	public function getFrameRate() : Number
	{
		return _nFrameRate;
	}
	
	public function setFrameRate(n:Number) : Void
	{
		_nFrameRate = n;
	}
	
	/**
	 * Get MSBeacon instance (Singleton pattern).<br><br>
	 * <strong>Usage :</strong> <em>var oFB:MSBeacon = MSBeacon.getInstance();</em>
	 */
	public static function getInstance() : MSBeacon
	{
		if (!_oB) _oB = new MSBeacon();
		return _oB;
	}
	
	/**
	 * Destroy MSBeacon instance (Singleton pattern).<br><br>
	 * <strong>Usage :</strong> <em>MSBeacon.release();</em>
	 */
	public static function release() : Void
	{
		_oB.stop();
		delete _oB;
	}
	
	/**
	 * Starts the process.<br><br>
	 * <strong>Usage :</strong> <em>MSBeacon.getInstance().start();</em>
	 */
	public function start() : Void
	{
		clearInterval( _nID ); // security
		if (_nFrameRate == undefined) _nFrameRate = MSBeacon.DEFAULTFRAMERATE;
		_nID = setInterval(this, 'fireFrameEvent', _nFrameRate);
		_bIsP = true;
	}
	
	/**
	 * Stops the process.<br><br>
	 * <strong>Usage :</strong> <em>MSBeacon.getInstance().stop();</em>
	 */
	public function stop() : Void
	{
		clearInterval( _nID );
		_bIsP = false;
	}
	
	/**
	 * To query if MSBeacon's running.<br><br>
	 * <strong>Usage :</strong> <em>var b:Boolean = MSBeacon.getInstance().isPlaying;</em>
	 * @return Boolean.
	 */
	public function isPlaying() : Boolean
	{
		return _bIsP;
	}
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function addFrameListener(oL:IFrameListener) : Void
	{
		if (_listeners.length < 1) start();
		addListener(oL);
	}
	
	/**
	 * Add listener for receiving all events.<br><br>
	 * <strong>Usage :</strong> <em>t.addListener(myListener);</em>
	 * @param oL Listener object which implements IFrameListener interface.
	 */
	public function removeFrameListener(oL:IFrameListener) : Void
	{
		removeListener(oL);
		if (_listeners.length < 1) this.stop();
	}
	
	public function toString() : String
	{
		return 'MSBeacon' + HashCodeFactory.getKey( this );
	}
}