/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;
import com.bourre.medias.sound.SoundFactory;

class com.bourre.medias.sound.MixSoundFactory 
	extends SoundFactory
{
	private var _nVolume:Number;
	private var _nPan:Number;
	private var _oGain:Object;
	
	public static var onSoundCompleteEVENT:EventType = new EventType("onSoundComplete");

	public function MixSoundFactory() 
	{
		super();
		
		_nVolume = 127;
		_nPan =0;
		_oGain = new Object();
	}
	
	// Overidding
	public function playSoundLoop(sID:String) : Void
	{
		var snd:Sound = getSound( sID );
		snd.onSoundComplete = Delegate.create(this, _onSoundLoop, snd);
		snd.start(0, 1);
	}
	
	private function _onSoundLoop(snd:Sound) : Void
	{
		snd.start(0, 1);
		_fireOnSoundComplete();
	}
	
	private function _fireOnSoundComplete() : Void
	{
		_oEB.broadcastEvent( new BasicEvent(onSoundCompleteEVENT) );
	}
	
	public function removeSound(sID:String) : Void
	{
		super.removeSound(sID);
		delete _oGain[sID];
	}

	public function addSound(linkageID:String) : String
	{
		if (!_bIsInitialized) _init();
		_oSounds[ linkageID ] = _makeSound(linkageID, _nDepth);
		
		_oGain[ linkageID ] = 90;
		_adjustVolume( linkageID );
		
		_nDepth++;
		return linkageID;
	}
	
	public function setGain(sID:String, nGain:Number) : Void
	{
		if (nGain > 127) nGain = 127;
		if (nGain < 0) nGain = 0;
		_oGain[sID] = nGain;
		
		_adjustVolume(sID);
	}
	
	public function setAllGain(nGain:Number) : Void
	{
		if (nGain > 127) nGain = 127;
		if (nGain < 0) nGain = 0;
		for (var sID in _oSounds)
		{
			_oGain[sID] = nGain;
			_adjustVolume( sID );
		}
	}
	
	private function _adjustVolume(sID:String) : Void
	{
		var v:Number = _calculVolume( getGain(sID) );
		getSound(sID).setVolume( v );
	}
	
	private function _calculVolume(nGain:Number): Number
	{
		return (nGain / 100) * _nVolume;
	}
	
	public function getGain(sID:String) : Number
	{
		return _oGain[sID];
	}
	
	public function setVolume(n:Number) : Void
	{
		if (n > 100) n = 100;
		if (n < 0) n = 0;

		_nVolume = n;
		for (var sID:String in _oSounds) _adjustVolume( sID );
	}
	
	public function setPan(n:Number) : Void
	{
		if (n > 100) n = 100;
		if (n < -100) n = -100;

		_nPan = n;
		for (var sID in _oSounds) _oSounds[ sID ].setPan(_nPan);
	}
	
	public function getVolume() : Number
	{
		return _nVolume;
	}
	
	public function getPan() : Number
	{
		return _nPan;
	}
	
	public function toString() : String
	{
		return 'MixSoundFactory' + HashCodeFactory.getKey( this );
	}
}