/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;

class com.bourre.events.NumberEvent extends BasicEvent
{
	private var _n:Number;
	
	/**
	 * Class inherited from BasicEvent with Number property bonus.<br><br>
	 * <strong>Usage :</strong> <em>var e:NumberEvent = new NumberEvent(MyClass.onSomething, 12);</em>
	 * @param e event type (event name).
	 * @param n Number.
	 */
	public function NumberEvent(e:EventType, n:Number)
	{
		super(e);
		_n = n;
	}
	
	/**
	 * Get Number value.<br><br>
	 * <strong>Usage :</strong> <em>var n:Number = e.getNumber();</em>
	 * @return Number
	 */
	public function getNumber() : Number
	{
		return _n;
	}
	
	/**
	 * Set Number value.<br><br>
	 * <strong>Usage :</strong> <em>e.setNumber(12);</em>
	 * @param n Number
	 */
	public function setNumber(n:Number) : Void
	{
		_n = n;
	}
	
	public function toString() : String
	{
		return '[NumberEvent' + HashCodeFactory.getKey( this ) + ' : ' + getType() + ', ' + getNumber() + ']';
	}
}