/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;
import com.bourre.structures.Point;

class com.bourre.events.PointEvent extends BasicEvent
{
	private var _p:Point;
	
	/**
	 * Class inherited from BasicEvent with Point property bonus.<br><br>
	 * <strong>Usage :</strong> <em>var e:PointEvent = new PointEvent(MyClass.onSomething, p);</em>
	 * @param e event type (event name).
	 * @param p Point.
	 */
	public function PointEvent(e:EventType, p:Point)
	{
		super(e);
		_p = p;
	}
	
	/**
	 * Get Point value.<br><br>
	 * <strong>Usage :</strong> <em>var n:Point = e.getPoint();</em>
	 * @return Point
	 */
	public function getPoint() : Point
	{
		return _p;
	}
	
	/**
	 * Set Point value.<br><br>
	 * <strong>Usage :</strong> <em>e.setPoint(p);</em>
	 * @param n Point
	 */
	public function setPoint(p:Point) : Void
	{
		_p = p;
	}
	
	public function toString() : String
	{
		return '[PointEvent' + HashCodeFactory.getKey( this ) + ' : ' + getType() + ', ' + getPoint() + ']';
	}
}