/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */
 
import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.log.LogEvent;
import com.bourre.log.LogListener;

class com.bourre.utils.SosTracer
	implements LogListener
{
	private var _oXMLSocket:XMLSocket;
	private var _bIsConnected:Boolean;
	private var _aBuffer:Array;
	
	private static var _oInstance:SosTracer;
	
	public static var IP:String = "localhost";
	public static var PORT:Number = 4444;
	
	private function SosTracer()
	{
		_aBuffer = new Array();
		_oXMLSocket = new XMLSocket();
		_oXMLSocket.onConnect = Delegate.create(this, _onConnect);
		_oXMLSocket.connect (SosTracer.IP, SosTracer.PORT);
	}
	
	/**
	 * static methods
	 */
	public static function getInstance() : SosTracer
	{
		if (!SosTracer._oInstance) SosTracer._oInstance = new SosTracer();
		return SosTracer._oInstance;
	}

	/**
	 * instance methods
	 */
	public function clearOutput() : Void
	{
		_oXMLSocket.send (chr(1)+"\n");
	}
	
	public function output( o ) : Void
	{
		var s:String = getTimer() + ">>" + String(o) + "\n";
		if (_bIsConnected)
		{
			_output( s );
		} else
		{
			_buffer( s );
		}
	}
	
	/**
	 * private methods
	 */
	private function _buffer( s:String ) : Void
	{
		_aBuffer.push( s );
	}
	
	private function _output( s:String ) : Void
	{
		_oXMLSocket.send( s );
	}
	
	private function _onConnect() : Void
	{
		_emptyBuffer();
		_bIsConnected = true;
	}
	
	private function _emptyBuffer() : Void
	{
		var l:Number = _aBuffer.length;
		for (var i:Number = 0; i<l; i++) _output( _aBuffer[i] );
	}
	
	/**
	 * specific callbacks
	 */
	// Unity2 callbacks
	public function update(o, infoObj) : Void
	{
		output((infoObj.getLevel() + ": ") + infoObj.getMessage());
	}
	
	// LogListener callbacks
	public function onLog( e:LogEvent ) : Void
	{
		output( e );
	}
	
	/**
	 * Returns the string representation of this instance.
	 * @return the string representation of this instance
	 */
	public function toString() : String
	{
		return 'SosTracer' + HashCodeFactory.getKey( this );
	}
}
