/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.events.DynBasicEvent;
import com.bourre.events.EventBroadcaster;
import com.bourre.events.EventType;

class com.bourre.medias.sound.SoundFactory
{
	private var _oEB:EventBroadcaster;

	private var _bIsInitialized:Boolean;
	private var _oSounds:Object;
	private var _nDepth:Number;
	private var _oContainer:MovieClip;
	private var _bIsOn:Boolean;
	private var _noSound:Sound;
	
	public static var onSoundLoadEVENT:EventType = new EventType("onSoundLoad");

	public function SoundFactory() 
	{
		_oEB = new EventBroadcaster( this );
		
		_bIsInitialized = false;
		_oSounds = new Object();
		_nDepth = 1;
		_bIsOn = true;
	}
	
	public function addListener(oL) : Void
	{
		_oEB.addListener(oL);
	}
	
	public function addEventListener(e:EventType, oL) : Void
	{
		_oEB.addEventListener(e, oL);
	}
	
	public function removeListener(oL) : Void
	{
		_oEB.removeListener(oL);
	}
	
	public function removeEventListener(e:EventType, oL) : Void
	{
		_oEB.removeEventListener(e, oL);
	}
	
	private function _init(mcTarget:MovieClip) : Void
	{
		_oContainer = (mcTarget == undefined) ? _level0.createEmptyMovieClip("__snd__", 65535) : mcTarget;
		_noSound = _makeSound("", 0);
		_bIsInitialized = true;
	}
	
	private function _makeSound(sLinkageID:String, nSoundID:Number, sURL:String) : Sound
	{
		var container:MovieClip = _oContainer.createEmptyMovieClip("sound" + sLinkageID, nSoundID);
		var snd:Sound = new Sound(container);

		if (sURL)
		{
			var e:DynBasicEvent = new DynBasicEvent( SoundFactory.onSoundLoadEVENT );
			e.soundID = nSoundID;
			e.soundURL = sURL;
			snd.onLoad = Delegate.create(this, _fireEvent, e);
			snd.loadSound(sURL, false);

		} else
		{
			snd.attachSound(sLinkageID);
		}
		return snd;
	}
	
	private function _fireEvent(b:Boolean, e:DynBasicEvent) : Void
	{
		e.success = b;
		_oEB.broadcastEvent( e );
	}
		
	public function init(mcTarget:MovieClip) : Void 
	{ 
		_init(mcTarget); 
	}
	
	public function getSound(sID:String) : Sound 
	{ 
		return _bIsOn ? _oSounds[sID] : _noSound;
	}
	
	public function playSoundLoop(sID:String) : Void
	{
		getSound(sID).start(0,65535);
	}
	
	public function loadSound(sURL:String) : String
	{
		if (!_bIsInitialized) _init();
		var linkageID:String = sURL.split(".")[0];
		_oSounds[linkageID] = _makeSound(linkageID, _nDepth, sURL);
		_nDepth++;
		return linkageID;
	}
	
	public function removeSound(sID:String) : Void
	{
		var mc:MovieClip = _oContainer["sound" + sID ];
		mc.removeMovieClip();
		delete _oSounds[sID];
	}

	public function addSound(linkageID:String) : String
	{
		if (!_bIsInitialized) _init();
		_oSounds[ linkageID ] = _makeSound(linkageID, _nDepth);
		_nDepth++;
		return linkageID;
	}
	
	public function addSounds(a:Array) : Void
	{
		var l:Number = a.length;
		for (var c:Number=0; c<l ; c++) addSound(a[c]);
	}
	
	public function toggleOnOff() : Void 
	{ 
		if (_bIsOn)
		{
			goOff();
		} else
		{
			goOn();
		}
	}
	
	public function get isOn() : Boolean 
	{ 
		return _bIsOn; 
	}
	
	public function goOn() : Void 
	{ 
		_bIsOn = true; 
	}
	
	public function goOff() : Void 
	{ 
		_bIsOn = false;
		 for (var sID:String in _oSounds) _oSounds[ sID ].stop();
	}
	
	public function getAllSounds() : Array
	{
		var a:Array = new Array();
		for (var p:String in _oSounds) a.push( _oSounds[p] );
		return a;
	}
	
	public function clear() : Void
	{
		_oEB.removeAllListeners();

		delete _oSounds;
		_oContainer.removeMovieClip();
		_bIsInitialized = false;
	}
	
	public function toString() : String
	{
		return 'SoundFactory' + HashCodeFactory.getKey( this );
	}
}