/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.core.HashCodeFactory;
import com.bourre.events.BasicEvent;
import com.bourre.events.EventType;

class com.bourre.events.StringEvent extends BasicEvent
{
	private var _s:String;
	
	/**
	 * Class inherited from BasicEvent with String property bonus.<br><br>
	 * <strong>Usage :</strong> <em>var e:BooleanEvent = new StringEvent(MyClass.onSomething, true);</em>
	 * @param e event type (event name).
	 * @param s String.
	 */
	public function StringEvent(e:EventType, s:String)
	{
		super(e);
		_s = s;
	}
	
	/**
	 * Get String value.<br><br>
	 * <strong>Usage :</strong> <em>var s:String = e.getString();</em>
	 * @return String
	 */
	public function getString() : String
	{
		return _s;
	}
	
	/**
	 * Set String value.<br><br>
	 * <strong>Usage :</strong> <em>e.setString('hello');</em>
	 * @param s String
	 */
	public function setString(s:String) : Void
	{
		_s = s;
	}
	
	public function toString() : String
	{
		return 'StringEvent' + HashCodeFactory.getKey( this ) + ' : ' + getType() + ', ' + getString();
	}
}