/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 1.0
 */

import com.bourre.events.*;
import com.bourre.transitions.*;

class com.bourre.transitions.TweenEvent extends BasicEvent
{
	private var _oTween:ITween;
	
	/**
	 * Event broadcasted by ITween family classes.<br><br>
	 * <strong>Usage :</strong> <em>var e:TweenEvent = new TweenEvent(TweenFPS.onMotionFinished, this);</em>
	 * @param e event type (event name).
	 * @param oTween event source (tween).
	 */
	public function TweenEvent(e:TweenEventType, oTween:ITween)
	{
		super(e);
		_oTween = oTween;
	}
	
	/**
	 * Get tween event source.<br><br>
	 * <strong>Usage :</strong> <em>var t:TweenMS = TweenMS(e.getType());</em>
	 * @return ITween
	 */
	public function getTween() : ITween
	{
		return _oTween;
	}
	
	public function toString() : String
	{
		return '[TweenEvent : ' + getType() + ', ' + getTween() + ']';

	}
}