/*
 * Copyright the original author or authors.
 * 
 * Licensed under the MOZILLA PUBLIC LICENSE, Version 1.1 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.mozilla.org/MPL/MPL-1.1.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 
 /**
 * @author Francis Bourre
 * @version 3.0
 */

import com.bourre.commands.Delegate;
import com.bourre.core.HashCodeFactory;
import com.bourre.data.libs.AbstractLib;
import com.bourre.data.libs.XMLToObjectEvent;
import com.bourre.events.EventType;
import com.bourre.log.Logger;
import com.bourre.log.LogLevel;
import com.bourre.structures.Point;

class com.bourre.data.libs.XMLToObject extends AbstractLib
{
	private var _oTarget;
	
	public static var onLoadInitEVENT:EventType = AbstractLib.onLoadInitEVENT;
	public static var onLoadProgressEVENT:EventType = AbstractLib.onLoadProgressEVENT;
	public static var onTimeOutEVENT:EventType = AbstractLib.onTimeOutEVENT;
 	
 	private static function _stripSpaces(s:String) : String 
	{
        var i:Number = 0;
		while(i < s.length && s.charCodeAt(i) == 32) i++;
		var j:Number = s.length-1;
		while(j > -1 && s.charCodeAt(j) == 32) j--;
        return s.substr(i, j-i+1);
  	}
  	
  	private static function _split(sE:String) : Array
  	{
  		var b:Boolean = true;
  		var a:Array = new Array();
  		var l:Number = sE.length;
  		var n:Number;
  		var s:String = '';
  		
  		for (var i:Number = 0; i<l; i++)
  		{
  			var c:Number = sE.charCodeAt(i);
  			if ( b && (c == 34 || c == 39))
  			{
  				b = false;
  				n = c;
  			} else if (!b && n == c)
  			{
  				b = true;
  				n = undefined;
  			}
  			
  			if (c == 44 && b)
  			{
   				a.push(s);
  				s = '';
  			} else
  			{
  				s += sE.substr(i, 1);	
  			}
  		}
  		a.push(s);
  		return a;
  	}
 	
 	private static function _explode(sE:String) : Array 
	{
  		var t:Array = XMLToObject._split(sE);
  		
  		var aR:Array = new Array();
  		var l:Number = t.length;
  		for (var y=0; y<l; y++) 
		{
			var s:String = XMLToObject._stripSpaces(t[y]);
			if (s == 'true' || s == 'false')
			{
				aR.push(Boolean(s));
			} else
			{
				if (s.charCodeAt(0) == 34 || s.charCodeAt(0) == 39)
				{
					aR.push( s.substr(1, s.length-2) );
				} else
				{
					aR.push( Number(s) );
				}
			}
		}
 		return aR;
  	}
  	
  	private static function _getUniquePropName(o, s:String) : String
  	{
  		var n:Number = -1;
  		var sName:String = s;
  		
  		while (o[sName]) sName = s + String( ++n );
  		return sName;
  	}
	
	private static function _deserializeData(oXML:XMLNode, oL) : Object
	{
		var o:Object = new Object();
    	for (var x = 0; x < oXML.childNodes.length; x++) 
		{
			var sName:String = oXML.childNodes[x].attributes.name;
			if (sName == undefined) sName = oXML.childNodes[x].nodeName;
					
			if ( o[sName] ) 
			{
				Logger.LOG( "XMLToObject parsed two nodes with identical names : '" + sName + "'.", LogLevel.WARN);
				sName = _getUniquePropName(o, sName);
			} 
			
			o[sName] = {};
                   
            var sType:String = oXML.childNodes[x].attributes.type;
            if (sType == undefined) sType = "xml2o";
			o[sName]["varType"] = sType;
			o[sName]["data"] = (sType != "xml2o") ? oXML.childNodes[x].firstChild.nodeValue : oXML.childNodes[x];

		}
		
        for(var x:String in o) 
		{

			switch (o[x].varType)
			{
                              		case "number" :
                              		oL[x] = Number( XMLToObject._stripSpaces(o[x].data) );
                              		break;
                              		case "string" :
                              		oL[x] = o[x].data;
                              		break;
                              		case "array" :
                              		oL[x] = XMLToObject._explode(o[x].data);
                              		break;
                              		case "boolean" :
                              		oL[x] = (XMLToObject._stripSpaces(o[x].data) == "true" || Number( XMLToObject._stripSpaces(o[x].data) ) == 1);
                              		break;
									
									// Custom objects deserialization examples
									case "xml2o" :
									oL[x] = XMLToObject._deserializeData(o[x].data, {});
									break;
									
									case "point" :
									var t = XMLToObject._explode(o[x].data);
									oL[x] = new Point(t[0], t[1]);
									break;
									
									//
      								case "default" :
      								// do nothing, specify something if u want ...
      								Logger.LOG('Warning, ' + o[x].varType + ' type is not supported!', LogLevel.WARN);
      								break;
			}
		}
		return oL;
	}
	
	private function _onLoadXML(bSuccess:Boolean) : Void
	{
		var oXML = getContent();
		var sURL:String = this.getURL();
		
		if (bSuccess) 
		{
    		if (oXML.status == 0)
    		{
    			readData();
    		} else
    		{
    			Logger.LOG('XML was loaded successfully, but was unable to be parsed : ' + sURL, LogLevel.ERROR);
    		}
    		
    		switch (oXML.status) 
    		{
   				 case 0 :
      				Logger.LOG('XML parsing was completed successfully' + ' : ' + sURL, LogLevel.INFO);
      				break;
    			case -2 :
      				Logger.LOG('A CDATA section was not properly terminated' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -3 :
      				Logger.LOG('The XML declaration was not properly terminated' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -4 :
      				Logger.LOG('The DOCTYPE declaration was not properly terminated' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -5 :
      				Logger.LOG('A comment was not properly terminated' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -6 :
      				Logger.LOG('An XML element was malformed' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -7 :
      				Logger.LOG('Out of memory' + ' : ' + sURL, LogLevel.FATAL);
      				break;
    			case -8 :
      				Logger.LOG('An attribute value was not properly terminated' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -9 :
     				 Logger.LOG('A start-tag was not matched with an end-tag' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			case -10 :
      				Logger.LOG('An end-tag was encountered without a matching start-tag' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    			default :
      				Logger.LOG('An unknown error has occurred' + ' : ' + sURL, LogLevel.ERROR);
      				break;
    		}
    		
  		} else 
  		{
    		Logger.LOG('Unable to load/parse XML : ' + sURL, LogLevel.ERROR);
  		}
	}
	
	/*
	 *
	 */

	public function XMLToObject(o) 
	{
		super( new XML() );
		
		setContent( new XML() );
		if (o != undefined) setObject( o );
	}
	
	public function initEventSource() : Void
	{
		_e = new XMLToObjectEvent(null, this);
	}
	
	public function onLoadInit() : Void
	{
		// overwriting for delaying 'onLoadInit' broadcast
	}

	public function readData() : Void
	{
		XMLToObject._deserializeData(getContent().firstChild, getObject());
    	super.onLoadInit();
	}
 
 	public function load(sURL:String) : Void
	{
		if (sURL != undefined) setURL( sURL );
		
		if (getObject() == undefined) 
		{
			Logger.LOG("**Warning** You must specify an object to decorate with 'setObject' method before using 'load' method with XMLToObject", LogLevel.WARN);
			setObject( new Object() );
		}
		
  		getContent().ignoreWhite = true;
  		getContent().onLoad = Delegate.create(this, _onLoadXML);
  		getContent().load( this.getURL() );
  		super.load();
  	}
	
	public function toString() : String
	{
		return 'XMLToObject' + HashCodeFactory.getKey( this );
	}
	
	// XML to deserialize
	public function setContent(oXML:XML) : Void
	{
		super.setContent( oXML );
	}
	
	public function getContent() : XML
	{
		return XML( super.getContent() );
	}
	
	// Object to decorate
	public function setObject(o) : Void
	{
		_oTarget = o;
	}
	
	public function getObject()
	{
		return _oTarget;
	}
}